/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -a singleapplicationadaptor -i lxqtsingleapplication.h -l LXQt::SingleApplication /usr/src/ports/liblxqt/liblxqt-0.12.0-1.x86_64/src/liblxqt-0.12.0/dbus/org.lxqt.SingleApplication.xml
 *
 * qdbusxml2cpp is Copyright (C) 2017 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "singleapplicationadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class SingleApplicationAdaptor
 */

SingleApplicationAdaptor::SingleApplicationAdaptor(LXQt::SingleApplication *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

SingleApplicationAdaptor::~SingleApplicationAdaptor()
{
    // destructor
}

void SingleApplicationAdaptor::activateWindow()
{
    // handle method call org.lxqt.SingleApplication.activateWindow
    parent()->activateWindow();
}


#include "singleapplicationadaptor.moc"
